/*
 * Projeto: SGER - Sistema Gestor de Empresas Rodovirias
 *
 * Tipo: Fachada
 *
 * Marca Registrada (c) 2002 ADFT
 * Grupo do Centro de Informtica da UFPE, Recife-PE
 * Todos os direitos reservados.
 */

package gestao.rodoviario.fachada;

import gestao.rodoviario.financas.Financas;
import gestao.rodoviario.funcionario.Funcionario;
import gestao.rodoviario.funcionario.CadastroDeFuncionario;
import gestao.rodoviario.passageiro.Passageiro;
import gestao.rodoviario.passageiro.CadastroDePassageiro;
import gestao.rodoviario.passagem.Passagem;
import gestao.rodoviario.passagem.CadastroDePassagem;
import gestao.rodoviario.viagem.Viagem;
import gestao.rodoviario.viagem.CadastroDeViagem;
import gestao.rodoviario.onibus.Onibus;
import gestao.rodoviario.onibus.CadastroDeOnibus;

import gestao.rodoviario.excecoes.ObjetoInexistenteException;
import gestao.rodoviario.excecoes.ObjetoJaCadastradoException;
import gestao.rodoviario.excecoes.ObjetoVazioException;


/**
 * Fachada  a classe que reune todas as operacoes do sistema. Permitindo
 * posterior acesso pela GUI
 *
 * Exemplo de uso:
 * Fachada fachada = new Fachada();
 *
 * @author  ADFT
 */
public class Fachada {

    private Financas financas;
    private CadastroDeFuncionario cadastroDeFuncionario;
    private CadastroDePassageiro cadastroDePassageiro;
    private CadastroDePassagem cadastroDePassagem;
    private CadastroDeViagem cadastroDeViagem;
    private CadastroDeOnibus cadastroDeOnibus;

   /**
    * Construtor da classe Fachada
    */
    public Fachada() {
        financas              = new Financas();
        cadastroDeFuncionario = new CadastroDeFuncionario();
        cadastroDePassageiro  = new CadastroDePassageiro();
        cadastroDePassagem    = new CadastroDePassagem();
        cadastroDeViagem      = new CadastroDeViagem();
        cadastroDeOnibus      = new CadastroDeOnibus();
    }

   /**
    * Retorna as finanas do sistema
    *
    * @return   Financas
    */
    public Financas getFinancas() {
        return this.financas;
    }

   /**
    * Seta as finanas do sistema
    *
    * @param   Financas
    */
    public void setFinancas(){
        this.financas = new Financas(0,0,0,cadastroDePassagem,cadastroDeViagem);

    }

   /**
    * Insere um onibus na frota da empresa
    *
    * @param   Onibus
    */
    public void inserirOnibus(Onibus onibus)
        throws ObjetoJaCadastradoException, ObjetoVazioException {
        this.cadastroDeOnibus.inserirOnibusNaFrota(onibus);
    }

   /**
    * Remove um onibus na frota da empresa
    *
    * @param   Onibus
    */
    public void removerOnibus(Onibus onibus)
        throws ObjetoInexistenteException, ObjetoVazioException {
        this.cadastroDeOnibus.removerOnibusDaFrota(onibus);
    }

   /**
    * Consulta um onibus da frota, caso ele exista
    *
    * @param   Onibus
    *
    * @return   Onibus[]
    */
    public Onibus[] consultarOnibus(Onibus onibus)
        throws ObjetoJaCadastradoException, ObjetoInexistenteException, ObjetoVazioException {
        return (this.cadastroDeOnibus.consultarOnibusDaFrota(onibus));
    }

   /**
    * Retorna todos os onibus da frota
    *
    * @param   Onibus
    *
    * @return  Onibus[]
    */
    public Onibus[] listarTodosOnibus(){
        return (this.cadastroDeOnibus.getTodosOnibusDaFrota());
    }

   /**
    * Insere um funcionario na empresa
    *
    * @param   Funcionario
    */
    public void inserirFuncionario(Funcionario funcionario)
        throws ObjetoJaCadastradoException, ObjetoVazioException {
        this.cadastroDeFuncionario.inserirFuncionarioNaListaDeFuncionario(funcionario);
    }

   /**
    * Remove um funcionario da empresa
    *
    * @param   Funcionario
    */
    public void removerFuncionario(Funcionario funcionario)
        throws ObjetoInexistenteException, ObjetoVazioException {
        this.cadastroDeFuncionario.removerFuncionarioDaListaDeFuncionario(funcionario);
    }

    /**
    * Retorna todos os funcionarios da empresa
    *
    * @param   Funcionario
    *
    * @return Funcionario[]
    */
    public Funcionario[] consultarFuncionario(Funcionario funcionario)
        throws ObjetoJaCadastradoException, ObjetoInexistenteException, ObjetoVazioException {
        return (this.cadastroDeFuncionario.consultarFuncionarioDaListaDeFuncionario(funcionario));
    }

   /**
    * Retorna todos os funcionarios da empresa
    *
    * @param   Funcionario
    *
    * @return  Funcionario[]
    */
    public Funcionario[] listarTodosFuncionario(){
        return (this.cadastroDeFuncionario.getListaDeFuncionariosCadastrados());
    }

   /**
    * Insere passageiro na empresa
    *
    * @param   Passageiro
    */
    public void inserirPassageiro(Passageiro passageiro)
        throws ObjetoJaCadastradoException, ObjetoVazioException {
        this.cadastroDePassageiro.inserirPassageiroNaListaDePasageiro(passageiro);
    }

   /**
    * Remove passageiro da empresa
    *
    * @param   Passageiro
    */
    public void removerPassageiro(Passageiro passageiro)
        throws ObjetoInexistenteException, ObjetoVazioException {
        this.cadastroDePassageiro.removerPassageiroDaListaDePasageiro(passageiro);
    }

   /**
    * Retorna passageiro
    *
    * @param   Passageiro
    *
    * @return  Passageiro[]
    */
    public Passageiro[] consultarPassageiro(Passageiro passageiro)
        throws ObjetoJaCadastradoException, ObjetoInexistenteException, ObjetoVazioException {
        return (this.cadastroDePassageiro.consultarPassageiroDaListaDePasageiro(passageiro));
    }

   /**
    * Retorna todos os passageiros
    *
    * @param   Passageiro
    *
    * @return  Passageiro[]
    */
    public Passageiro[] listarTodosPassageiro(){
        return (this.cadastroDePassageiro.getListaDePassagirosCadastrados());
    }

   /**
    * Insere passagem
    *
    * @param   Passagem
    */
    public void inserirPassagem(Passagem passagem)
        throws ObjetoJaCadastradoException, ObjetoVazioException {
        this.cadastroDePassagem.inserirPassagemNaListaDePassagem(passagem);
    }

   /**
    * Remove passagem
    *
    * @param   Passagem
    */
    public void removerPassagem(Passagem passagem)
        throws ObjetoInexistenteException, ObjetoVazioException {
        this.cadastroDePassagem.removerPassagemDaListaDePassagem(passagem);
    }

   /**
    * Retorna passagem
    *
    * @param   Passagem
    *
    * @return  Passagem[]
    */
    public Passagem[] consultarPassagem(Passagem passagem)
        throws ObjetoJaCadastradoException, ObjetoInexistenteException, ObjetoVazioException {
        return (this.cadastroDePassagem.consultarPassagemDaListaDePassagem(passagem));
    }

   /**
    * Retorna todas as passagens
    *
    * @param   Passagem
    *
    * @return  Passagem[]
    */
    public Passagem[] listarTodosPassagem(){
       return (this.cadastroDePassagem.getListaDePassagensCadastradas());
    }

   /**
    * Insere viagem
    *
    * @param   Viagem
    */
    public void inserirViagem(Viagem viagem)
        throws ObjetoJaCadastradoException, ObjetoInexistenteException, ObjetoVazioException {
        this.cadastroDeViagem.inserirViagemNaListaDeViagem(viagem);
    }

   /**
    * Remove viagem
    *
    * @param   Viagem
    */
    public void removerViagem(Viagem viagem)
        throws ObjetoJaCadastradoException, ObjetoInexistenteException, ObjetoVazioException {
        this.cadastroDeViagem.removerViagemDaListaDeViagem(viagem);
    }

   /**
    * Retorna viagem
    *
    * @param   Viagem
    *
    * @return  Viagem[]
    */
    public Viagem[] consultarViagem(Viagem viagem)
        throws ObjetoJaCadastradoException, ObjetoInexistenteException, ObjetoVazioException {
        return (this.cadastroDeViagem.consultarViagemDaListaDeViagem(viagem));
    }

   /**
    * Retorna todas passagens
    *
    * @param   Viagem
    *
    * @return  Viagem[]
    */
    public Viagem[] listarTodosViagem(){
       return (this.cadastroDeViagem.getListaDeViagensCadastradas());
    }
}
