/*
 * Projeto: SGER - Sistema Gestor de Empresas Rodovirias
 *
 * Tipo: RepositorioDeFuncionario
 *
 * Marca Registrada (c) 2002 ADFT
 * Grupo do Centro de Informtica da UFPE, Recife-PE
 * Todos os direitos reservados.
 */

package gestao.rodoviario.funcionario;

import gestao.rodoviario.excecoes.ObjetoInexistenteException;
import gestao.rodoviario.excecoes.ObjetoJaCadastradoException;
import gestao.rodoviario.excecoes.ObjetoVazioException;


/**
 * RepositorioDeFuncionario  um tipo que representa o corpo de funcionrios
 * do sistema.
 *
 * Exemplo de uso:
 * RepositorioDeFuncionario funcionarios = new RepositorioDeFuncionario()
 *
 * @author  ADFT
 */
public class RepositorioDeFuncionario {

    private Funcionario funcionario;
    private RepositorioDeFuncionario proximo;

   /**
    * Construtor da classe RepositorioDeFuncionario
    */
    public RepositorioDeFuncionario() {
        this.funcionario = new Funcionario();
        this.proximo = null;
    }

   /**
    * Retorna o prximo funcionrio da lista
    *
    * @return   Um Funcionario
    */
    private RepositorioDeFuncionario getProximo() {
        return this.proximo;
    }

   /**
    * Insere funcionario no sistema
    *
    * @param    Um Funcionario
    */
    public void inserirFuncionario(Funcionario funcionario)
        throws ObjetoVazioException, ObjetoJaCadastradoException {

        if (!funcionario.isNull()) {
            if (!this.exite(funcionario)) {
                if (this.funcionario.isNull()) {
                    this.funcionario = new Funcionario(funcionario.getNome(),
                                                       funcionario.getIdentidade(),
                                                       funcionario.getCargoDoFuncionario());
                    this.proximo = new RepositorioDeFuncionario();
                } else {
                    this.proximo.inserirFuncionario(funcionario);
                }
            } else {
                throw new ObjetoJaCadastradoException("FUNCIONARIO","inserir");
            }
        } else {
            throw new ObjetoVazioException("FUNCIONARIO","inserir");
        }
    }

   /**
    * Remove funcionario do sistema
    *
    * @param    Um Funcionario
    */
    public boolean removerFuncionario(Funcionario funcionario)
        throws ObjetoInexistenteException, ObjetoVazioException {

        boolean resultado = false;
        if (!funcionario.isNull()) {
            if (this.funcionario.match(funcionario)) {
                resultado = true;
                this.funcionario = this.proximo.funcionario;
                this.proximo = this.proximo.proximo;
            } else {
                if (this.proximo != null) {
                    resultado = this.proximo.removerFuncionario(funcionario);
                } else {
                    throw new ObjetoInexistenteException("FUNCIONARIO","remover");
                }
            }
            return resultado;
        } else {
            throw new ObjetoVazioException("FUNCIONARIO","remover");
        }
    }

  /**
    * Mtodo que verifica a existencia de um funcionario no repositorio
    *
    * @param    Um funcionario
    *
    * @return   true se o elemento existe no sistema
    *           false caso contrrio
    */
    private boolean exite(Funcionario funcionario) {
            if (this.proximo != null) {
                if (this.funcionario.equals(funcionario)) {
                    return true;  // Encontrou
                } else {
                    return this.proximo.exite(funcionario);
                }
            } else {
                return false; // Fim da lista
            }
    }

  /**
    * Mtodo auxiliar para procura de funcionrio no sistema
    *
    * @param    Um funcionrio com os atributos de interesse diferentes de nulo
    * @param    O repositorio que acumula os funcionrios resultado da pesquisa
    */
    private void procurar(Funcionario funcionario, RepositorioDeFuncionario acumulador)
        throws ObjetoVazioException, ObjetoJaCadastradoException,ObjetoInexistenteException {

        if (!this.funcionario.isNull()) {
            if (this.funcionario.match(funcionario)) {
                acumulador.inserirFuncionario(this.funcionario);
            }
            this.proximo.procurar(funcionario,acumulador);
        }
    }

  /**
    * Procura funcionrios no sistema que casam alguns atributos com
    * os atributos do funcionrio de entrada
    *
    * @param    Um funcionrio com os atributos de interesse diferentes de nulo
    */
    public RepositorioDeFuncionario procurarFuncionario(Funcionario funcionario)
        throws ObjetoJaCadastradoException, ObjetoVazioException, ObjetoInexistenteException {

        if (funcionario.isNull()) {
          throw new ObjetoVazioException("FUNCIONARIO","procurar");
        }

        RepositorioDeFuncionario resposta = new RepositorioDeFuncionario();
        procurar (funcionario,resposta);

        // O repositorio retornou vazio da busca
        if (resposta.proximo == null) {
            throw new ObjetoInexistenteException("FUNCIONARIO","procurar");
        }

        return resposta;
    }

   /**
    * Permite o acesso por outras classes aos elementos do repositrio
    *
    * @return   Array com todos os elementos do repositrio
    */
    public Funcionario[] getIterador() {
        Funcionario resultado[];
        RepositorioDeFuncionario auxiliar;
        int quantidadeDeFuncionario = 0;
        int indice = 0;

        auxiliar = this;
        while(auxiliar.proximo != null) {
            quantidadeDeFuncionario = quantidadeDeFuncionario + 1;
            auxiliar = auxiliar.proximo;
        }

        // +1 para marcar com null o final do array
        resultado = new Funcionario[quantidadeDeFuncionario + 1];

        if (quantidadeDeFuncionario != 0) {
            auxiliar = this;
            while(auxiliar.proximo != null) {
                resultado[indice] = auxiliar.funcionario;
                auxiliar = auxiliar.proximo;
                indice = indice + 1;
            }
        }

        return resultado;
    }

}
