/*
 * Projeto: SGER - Sistema Gestor de Empresas Rodovirias
 *
 * Tipo: RepositorioDeOnibus
 *
 * Marca Registrada (c) 2002 ADFT
 * Grupo do Centro de Informtica da UFPE, Recife-PE
 * Todos os direitos reservados.
 */

package gestao.rodoviario.onibus;

import gestao.rodoviario.excecoes.ObjetoInexistenteException;
import gestao.rodoviario.excecoes.ObjetoJaCadastradoException;
import gestao.rodoviario.excecoes.ObjetoVazioException;


/**
 * RepositorioDeOnibus  um tipo que representa uma frota de nibus no sistema.
 *
 * Exemplo de uso:
 * RepositorioDeOnibus frota = new RepositorioDeOnibus()
 *
 * @author  ADFT
 */
public class RepositorioDeOnibus implements InterfaceDoRepositorioDeOnibus {

    private Onibus onibus;
    private RepositorioDeOnibus proximo;

   /**
    * Construtor da classe RepositorioDeOnibus
    */
    public RepositorioDeOnibus() {
        this.onibus = new Onibus();
        this.proximo = null;
    }

   /**
    * Retorna o prximo nibus da lista
    *
    * @return   Um RepositorioDeOnibus
    */
    private RepositorioDeOnibus getProximo() {
        return this.proximo;
    }

   /**
    * Insere um nibus na frota do sistema
    *
    * @param    Um Onibus
    */
    public void inserirOnibus(Onibus onibus)
        throws ObjetoVazioException, ObjetoJaCadastradoException {

        if (!onibus.isNull()) {
            if (!this.exite(onibus)) {
                if (this.onibus.isNull()) {
                    this.onibus = new Onibus(onibus.getModeloOnibus(),
                                             onibus.getDataRevOnibus(),
                                             onibus.getNumeroOnibus(),
                                             onibus.getAnoFabOnibus());
                    this.proximo = new RepositorioDeOnibus();
                } else {
                    this.proximo.inserirOnibus(onibus);
                }
            } else {
                throw new ObjetoJaCadastradoException("ONIBUS","inserir");
            }
        } else {
            throw new ObjetoVazioException("ONIBUS","inserir");
        }
    }

   /**
    * Remove um nibus da frota do sistema
    *
    * @param    Um nibus
    */
    public boolean removerOnibus(Onibus onibus)
        throws ObjetoInexistenteException, ObjetoVazioException {

        boolean resultado = false;
        if (!onibus.isNull()) {
            if (this.onibus.match(onibus)) {
                resultado = true;
                this.onibus = this.proximo.onibus;
                this.proximo = this.proximo.proximo;
            } else {
                if (this.proximo != null) {
                    resultado = this.proximo.removerOnibus(onibus);
                } else {
                    throw new ObjetoInexistenteException("ONIBUS","remover");
                }
            }
            return resultado;
        } else {
            throw new ObjetoVazioException("ONIBUS","remover");
        }
    }

  /**
    * Mtodo que verifica a existencia de um onibus no repositorio
    *
    * @param    Um onibus
    *
    * @return   true se o elemento existe no sistema
    *           false caso contrrio
    */
    private boolean exite(Onibus onibus) {
            if (this.proximo != null) {
                if (this.onibus.equals(onibus)) {
                    return true;  // Encontrou
                } else {
                    return this.proximo.exite(onibus);
                }
            } else {
                return false; // Fim da lista
            }
    }

   /**
    * Mtodo auxiliar para procura de nibus na frota do sistema
    *
    * @param    Um nibus com os atributos de interesse diferentes de nulo
    * @param    O repositorio que acumula os onibus resultado da pesquisa
    */
    private void procurar(Onibus onibus, RepositorioDeOnibus acumulador)
        throws ObjetoJaCadastradoException, ObjetoVazioException {

        if (!this.onibus.isNull()) {

            if (this.onibus.match(onibus)) {


                acumulador.inserirOnibus(this.onibus);
            }
            this.proximo.procurar(onibus,acumulador);
        }
    }

   /**
    * Procura nibus na frota do sistema que casam alguns atributos com
    * os atributos do nibus de entrada
    *
    * @param    Um nibus com os atributos de interesse diferentes de nulo
    */
    public RepositorioDeOnibus procurarOnibus(Onibus onibus)
        throws ObjetoJaCadastradoException, ObjetoVazioException, ObjetoInexistenteException {


        if (onibus.isNull()) {

          throw new ObjetoVazioException("ONIBUS","procurar");
        }


        RepositorioDeOnibus resposta = new RepositorioDeOnibus();
        procurar (onibus,resposta);

        // O repositorio retornou vazio da busca
        if (resposta.proximo == null) {
            throw new ObjetoInexistenteException("ONIBUS","procurar");
        }

        return resposta;
    }

   /**
    * Permite o acesso por outras classes aos elementos do repositrio
    *
    * @return   Array com todos os elementos do repositrio
    */
    public Onibus[] getIterador() {
        Onibus resultado[];
        RepositorioDeOnibus auxiliar;
        int quantidadeDeOnibus = 0;
        int indice = 0;

        auxiliar = this;
        while(auxiliar.proximo != null) {
            quantidadeDeOnibus = quantidadeDeOnibus + 1;
            auxiliar = auxiliar.proximo;
        }

        // +1 para marcar com null o final do array
        resultado = new Onibus[quantidadeDeOnibus + 1];

        if (quantidadeDeOnibus != 0) {
            auxiliar = this;
            while(auxiliar.proximo != null) {
                resultado[indice] = auxiliar.onibus;
                auxiliar = auxiliar.proximo;
                indice = indice + 1;
            }
        }

        return resultado;
    }

}
