/*
 * Projeto: SGER - Sistema Gestor de Empresas Rodovirias
 *
 * Tipo: RepositorioDePassageiro
 *
 * Marca Registrada (c) 2002 ADFT
 * Grupo do Centro de Informtica da UFPE, Recife-PE
 * Todos os direitos reservados.
 */

package gestao.rodoviario.passageiro;

import gestao.rodoviario.passageiro.PassageiroCrianca;
import gestao.rodoviario.passageiro.PassageiroIdoso;

import gestao.rodoviario.excecoes.ObjetoInexistenteException;
import gestao.rodoviario.excecoes.ObjetoJaCadastradoException;
import gestao.rodoviario.excecoes.ObjetoVazioException;


/**
 * RepositorioDePassageiro  um tipo que representa os passageiros no sistema
 *
 * Exemplo de uso:
 * RepositorioDePassageiro passageiros = new RepositorioDePassageiro()
 *
 * @author  ADFT
 */
public class RepositorioDePassageiro {

    private Passageiro passageiro;
    private RepositorioDePassageiro proximo;

   /**
    * Construtor da classe RepositorioDePassageiro
    */
    public RepositorioDePassageiro() {
     this.passageiro = new PassageiroCrianca();
     proximo = null;
    }

   /**
    * Insere um passageiro no sistema
    *
    * @param    Um Passageiro
    */
    public void inserirPassageiro(Passageiro passageiro)
        throws ObjetoVazioException, ObjetoJaCadastradoException {

        if (!passageiro.isNull()) {
            if (!this.exite(passageiro)) {
                if (this.passageiro.isNull()) {
                    if(passageiro instanceof PassageiroCrianca){
                        this.passageiro = passageiro;
                    }
                    if(passageiro instanceof PassageiroIdoso){
                        this.passageiro = passageiro;
                    }
                    if(passageiro instanceof PassageiroAdulto){
                        this.passageiro = passageiro;
                    }
                    this.proximo = new RepositorioDePassageiro();
                } else {
                    this.proximo.inserirPassageiro(passageiro);
                }
            } else {
                throw new ObjetoJaCadastradoException("PASSAGEIRO","inserir");
            }
        } else {
            throw new ObjetoVazioException("PASSAGEIRO","inserir");
        }
    }

   /**
    * Remove um passageiro do sistema
    *
    * @param    Um Passageiro
    */
    public boolean removerPassageiro(Passageiro passageiro)
        throws ObjetoInexistenteException, ObjetoVazioException {

        boolean resultado = false;
        if (!passageiro.isNull()) {
            if (this.passageiro.equals(passageiro)) {
                resultado = true;
                this.passageiro = this.proximo.passageiro;
                this.proximo = this.proximo.proximo;
            } else {
                if (this.proximo != null) {
                    resultado = this.proximo.removerPassageiro(passageiro);
                } else {
                    throw new ObjetoInexistenteException("PASSAGEIRO","remover");
                }
            }
            return resultado;
        } else {
            throw new ObjetoVazioException("PASSAGEIRO","remover");
        }
    }

  /**
    * Mtodo que verifica a existencia de um passageiro no repositorio
    *
    * @param    Um passageiro
    *
    * @return   true se o elemento existe no sistema
    *           false caso contrrio
    */
    private boolean exite(Passageiro passageiro) {
            if (this.proximo != null) {
                if (this.passageiro.equals(passageiro)) {
                    return true;  // Encontrou
                } else {
                    return this.proximo.exite(passageiro);
                }
            } else {
                return false; // Fim da lista
            }
    }

   /**
    * Mtodo auxiliar para procura de passageiro no sistema
    *
    * @param    Um passageiro com os atributos de interesse diferentes de nulo
    * @param    O repositorio que acumula os passageiros resultado da pesquisa
    */
    private void procurar(Passageiro passageiro, RepositorioDePassageiro acumulador)
        throws ObjetoJaCadastradoException, ObjetoVazioException {

        if (!this.passageiro.isNull()) {
            if (this.passageiro.match(passageiro)) {
                acumulador.inserirPassageiro(this.passageiro);
            }
            this.proximo.procurar(passageiro,acumulador);
        }
    }

   /**
    * Procura passageiros no sistema que casam alguns atributos com
    * os atributos do passageiro de entrada
    *
    * @param    Um passageiro com os atributos de interesse diferentes de nulo
    */
    public RepositorioDePassageiro procurarPassageiro(Passageiro passageiro)
        throws ObjetoJaCadastradoException, ObjetoVazioException, ObjetoInexistenteException {

        if (passageiro.isNull()) {
          throw new ObjetoVazioException("PASSAGEIRO","procurar");
        }

        RepositorioDePassageiro resposta = new RepositorioDePassageiro();
        procurar (passageiro,resposta);

        // O repositorio retornou vazio da busca
        if (resposta.proximo == null) {
            throw new ObjetoInexistenteException("PASSAGEIRO","procurar");
        }

        return resposta;
    }

   /**
    * Permite o acesso por outras classes aos elementos do repositrio
    *
    * @return   Array com todos os elementos do repositrio
    */
    public Passageiro[] getIterador() {
        Passageiro resultado[];
        RepositorioDePassageiro auxiliar;
        int quantidadeDePassageiro = 0;
        int indice = 0;

        auxiliar = this;
        while(auxiliar.proximo != null) {
            quantidadeDePassageiro = quantidadeDePassageiro + 1;
            auxiliar = auxiliar.proximo;
        }

        // +1 para marcar com null o final do array
        resultado = new Passageiro[quantidadeDePassageiro + 1];

        if (quantidadeDePassageiro != 0) {
            auxiliar = this;
            while(auxiliar.proximo != null) {
                resultado[indice] = auxiliar.passageiro;
                auxiliar = auxiliar.proximo;
                indice = indice + 1;
            }
        }

        return resultado;
    }

}
