/*
 * Projeto: SGER - Sistema Gestor de Empresas Rodovirias
 *
 * Tipo: RepositorioDePassagem
 *
 * Marca Registrada (c) 2002 ADFT
 * Grupo do Centro de Informtica da UFPE, Recife-PE
 * Todos os direitos reservados.
 */

package gestao.rodoviario.passagem;

import gestao.rodoviario.excecoes.ObjetoInexistenteException;
import gestao.rodoviario.excecoes.ObjetoJaCadastradoException;
import gestao.rodoviario.excecoes.ObjetoVazioException;


/**
 * RepositorioDePassagem  um tipo que representa as passagens no sistema
 *
 * Exemplo de uso:
 * RepositorioDePassagem passagens = new RepositorioDePassagem()
 *
 * @author  ADFT
 */
public class RepositorioDePassagem {

    private Passagem passagem;
    private RepositorioDePassagem proximo;

   /**
    * Construtor da classe RepositorioDePassageiro
    */
    public RepositorioDePassagem() {
        passagem = new Passagem();
        proximo =  null;
    }

  /**
    * Insere uma passagem no sistema
    *
    * @param    Uma Passagem
    */
    public void inserirPassagem(Passagem passagem)
        throws ObjetoVazioException, ObjetoJaCadastradoException {

        if (!passagem.isNull()) {
            if (!this.exite(passagem)) {
                if (this.passagem.isNull()) {
                    this.passagem = new Passagem(passagem.getData(),
                                                 passagem.getViagemDaPassagem(),
                                                 passagem.getViagem(),
                                                 passagem.getPoltrona(),
                                                 passagem.getPreco());
                    this.proximo = new RepositorioDePassagem();
                } else {
                    this.proximo.inserirPassagem(passagem);
                }
            } else {
                throw new ObjetoJaCadastradoException("PASSAGEM","inserir");
            }
        } else {
            throw new ObjetoVazioException("PASSAGEM","inserir");
        }
    }

  /**
    * Remove uma passagem do sistema
    *
    * @param    Uma Passagem
    */
    public boolean removerPassagem(Passagem passagem)
        throws ObjetoInexistenteException, ObjetoVazioException {

        boolean resultado = false;
        if (!passagem.isNull()) {
            if (this.passagem.equals(passagem)) {
                resultado = true;
                this.passagem = this.proximo.passagem;
                this.proximo = this.proximo.proximo;
            } else {
                if (this.proximo != null) {
                    resultado = this.proximo.removerPassagem(passagem);
                } else {
                    throw new ObjetoInexistenteException("PASSAGEM","remover");
                }
            }
            return resultado;
        } else {
            throw new ObjetoVazioException("PASSAGEM","remover");
        }
    }

  /**
    * Mtodo que verifica a existencia de uma passagem no repositorio
    *
    * @param    Uma passagem
    *
    * @return   true se o elemento existe no sistema
    *           false caso contrrio
    */
    private boolean exite(Passagem passagem) {
            if (this.proximo != null) {
                if (this.passagem.equals(passagem)) {
                    return true;  // Encontrou
                } else {
                    return this.proximo.exite(passagem);
                }
            } else {
                return false; // Fim da lista
            }
    }

  /**
    * Mtodo auxiliar para procura de passagem no sistema
    *
    * @param    Um passageiro com os atributos de interesse diferentes de nulo
    * @param    O repositorio que acumula as passagens resultado da pesquisa
    */
    private void procurar(Passagem passagem, RepositorioDePassagem acumulador)
        throws ObjetoJaCadastradoException, ObjetoVazioException {

        if (!this.passagem.isNull()) {
            if (this.passagem.match(passagem)) {
                acumulador.inserirPassagem(this.passagem);
                this.proximo.procurar(passagem,acumulador);
            }
        }
    }

  /**
    * Procura passagens no sistema que casam alguns atributos com
    * os atributos da passagem de entrada
    *
    * @param    Uma passagem com os atributos de interesse diferentes de nulo
    */
    public  RepositorioDePassagem procurarPassagem(Passagem passagem)
        throws ObjetoJaCadastradoException, ObjetoVazioException, ObjetoInexistenteException {

        if (passagem.isNull()) {
          throw new ObjetoVazioException("PASSAGEM","procurar");
        }

        RepositorioDePassagem resposta = new RepositorioDePassagem();
        procurar (passagem,resposta);

        // O repositorio retornou vazio da busca
        if (resposta.proximo == null) {
            throw new ObjetoInexistenteException("PASSAGEM","procurar");
        }

        return resposta;
    }

   /**
    * Permite o acesso por outras classes aos elementos do repositrio
    *
    * @return   Array com todos os elementos do repositrio
    */
    public Passagem[] getIterador() {
        Passagem resultado[];
        RepositorioDePassagem auxiliar;
        int quantidadeDePassagem = 0;
        int indice = 0;

        auxiliar = this;
        while(auxiliar.proximo != null) {
            quantidadeDePassagem = quantidadeDePassagem + 1;
            auxiliar = auxiliar.proximo;
        }

        // +1 para marcar com null o final do array
        resultado = new Passagem[quantidadeDePassagem + 1];

        if (quantidadeDePassagem != 0) {
            auxiliar = this;
            while(auxiliar.proximo != null) {
                resultado[indice] = auxiliar.passagem;
                auxiliar = auxiliar.proximo;
                indice = indice + 1;
            }
        }

        return resultado;
    }

/*
    public static void main(String args[])
        throws ObjetoJaCadastradoException, ObjetoVazioException, ObjetoInexistenteException {

        RepositorioDePassagem r = new RepositorioDePassagem();
        RepositorioDePassagem s;
        Passagem p;
        Passagem[] passagens;
        int i;

        try {
    
            // CADASTRA 4 VIAGENS
            p = new Passagem();

            // CAUSAS DA EXCECAO ObjetoVazioException
            // r.removerPassagem(p);
            // r.inserirPassagem(p);

            p.setViagemDaPassagem("RECIFE - FORTALEZA"); p.setData("10/10/2002");
            r.inserirPassagem(p);
            p.setViagemDaPassagem("RECIFE - FORTALEZA"); p.setData("12/10/2002");
            r.inserirPassagem(p);
            p.setViagemDaPassagem("RECIFE - TIMBAUBA"); p.setData("8/10/2002");
            r.inserirPassagem(p);
            p.setViagemDaPassagem("RECIFE - PETROLINA"); p.setData("15/10/2002");
            r.inserirPassagem(p);

            // CAUSA DA EXCECAO ObjetoInexistenteException
            // p.setViagemDaPassagem("RECIFE - TOKIO"); r.removerPassagem(p);

            // CAUSA DA EXCECAO ObjetoJaCasdastradoException
            // r.inserirPassagem(p);


            // MOSTRA AS PASSAGENS NAVEGANDO PELO REPOSITORIO RESULTADO DE UMA PROCURA
            p = new Passagem();
            p.setViagemDaPassagem("RECIFE - FORTALEZA");
            s = r.procurarPassagem(p);
            while (s.proximo != null) {
                System.out.println(s.passagem.getViagemDaPassagem() + "  " + s.passagem.getData());
                s = s.proximo;
            }

            // CADASTRA MAIS 2 PASSAGENS
            p = new Passagem();
            p.setViagemDaPassagem("GARANHUNS - RECIFE"); p.setData("10/10/2002");
            r.inserirPassagem(p);
            p.setViagemDaPassagem("CARUARU - RECIFE"); p.setData("11/10/2002");
            r.inserirPassagem(p);

            // MOSTRA AS PASSAGENS UTILIZANDO getIterador()
            System.out.println();
            passagens = r.getIterador();
            for (i=0; i<6; i++) {
                System.out.println(passagens[i].getViagemDaPassagem() + "  " + passagens[i].getData());
            }

            // REMOVE A PRIMEIRA E A ULTIMA DA LISTA
            p = new Passagem();
            p.setViagemDaPassagem("RECIFE - FORTALEZA"); p.setData("10/10/2002");
            r.removerPassagem(p);
            p.setViagemDaPassagem("CARUARU - RECIFE"); p.setData("11/10/2002");
            r.removerPassagem(p);

            // MOSTRA AS PASSAGENS UTILIZANDO getIterador()
            System.out.println();
            passagens = r.getIterador();
            for (i=0; i<4; i++) {
                System.out.println(passagens[i].getViagemDaPassagem() + "  " + passagens[i].getData());
            }

        }

        catch (ObjetoVazioException excecao) {
            System.out.println(excecao.getMensagem());
        }

        catch (ObjetoInexistenteException excecao) {
            System.out.println(excecao.getMensagem());
        }

        catch (ObjetoJaCadastradoException excecao) {
            System.out.println(excecao.getMensagem());
        }

    }
*/

}
