/*
 * Projeto: SGER - Sistema Gestor de Empresas Rodovirias
 *
 * Tipo: RepositorioDeViagem
 *
 * Marca Registrada (c) 2002 ADFT
 * Grupo do Centro de Informtica da UFPE, Recife-PE
 * Todos os direitos reservados.
 */

package gestao.rodoviario.viagem;

import gestao.rodoviario.excecoes.ObjetoInexistenteException;
import gestao.rodoviario.excecoes.ObjetoJaCadastradoException;
import gestao.rodoviario.excecoes.ObjetoVazioException;


/**
 * RepositorioDeViagem  um tipo que representa as viagens no sistema.
 *
 * Exemplo de uso:
 * RepositorioDeViagem viagens = new RepositorioDeViagem()
 *
 * @author  ADFT
 */
public class RepositorioDeViagem {

    private Viagem viagem;
    private RepositorioDeViagem proximo;

   /**
    * Construtor da classe RepositorioDeViagem
    */
    public RepositorioDeViagem() {
        this.viagem = new Viagem();
        this.proximo = null;
    }

   /**
    * Insere uma viagem no sistema
    *
    * @param    Uma Viagem
    */
    public void inserirViagem(Viagem viagem)
        throws ObjetoVazioException, ObjetoJaCadastradoException {

        if (!viagem.isNull()) {
            if (!this.exite(viagem)) {
                if (this.viagem.isNull()) {
                    this.viagem = new Viagem(viagem.getEmbarque(),
                                             viagem.getDesembarque(),
                                             viagem.getOnibusDaViagem(),
                                             viagem.getHoraPartida(),
                                             viagem.getHoraChegada(),
                                             viagem.getDistancia(),
                                             viagem.getTempoDeViagem(),
                                             viagem.getCustoDaViagem());
                    this.proximo = new RepositorioDeViagem();
                } else {
                    this.proximo.inserirViagem(viagem);
                }
            } else {
                throw new ObjetoJaCadastradoException("VIAGEM","inserir");
            }
        } else {
            throw new ObjetoVazioException("VIAGEM","inserir");
        }
    }

   /**
    * Remove uma viagem do sistema
    *
    * @param    Uma Viagem
    */
    public boolean removerViagem(Viagem viagem)
        throws ObjetoInexistenteException, ObjetoVazioException {

        boolean resultado = false;
        if (!viagem.isNull()) {
            if (this.viagem.equals(viagem)) {
                resultado = true;
                this.viagem = this.proximo.viagem;
                this.proximo = this.proximo.proximo;
            } else {
                if (this.proximo != null) {
                    resultado = this.proximo.removerViagem(viagem);
                } else {
                    throw new ObjetoInexistenteException("VIAGEM","remover");
                }
            }
            return resultado;
        } else {
            throw new ObjetoVazioException("VIAGEM","remover");
        }
    }

  /**
    * Mtodo que verifica a existencia de uma viagem no repositorio
    *
    * @param    Uma viagem
    *
    * @return   true se o elemento existe no sistema
    *           false caso contrrio
    */
    private boolean exite(Viagem viagem) {
            if (this.proximo != null) {
                if (this.viagem.equals(viagem)) {
                    return true;  // Encontrou
                } else {
                    return this.proximo.exite(viagem);
                }
            } else {
                return false; // Fim da lista
            }
    }

  /**
    * Mtodo auxiliar para pesquisa de viagem no sistema
    *
    * @param    Uma viagem com os atributos de interesse diferentes de nulo
    * @param    O repositorio que acumula as viagens resultado da pesquisa
    */
    private void procurar(Viagem viagem, RepositorioDeViagem acumulador)
        throws ObjetoJaCadastradoException, ObjetoVazioException {

        if (!this.viagem.isNull()) {
            if (this.viagem.match(viagem)) {
                acumulador.inserirViagem(this.viagem);
            }
            this.proximo.procurar(viagem, acumulador);
        }
    }

  /**
    * Pesquisa viagens no sistema que casam alguns atributos com
    * os atributos da viagem de entrada
    *
    * @param    Uma viagem com os atributos de interesse diferentes de nulo
    */
    public RepositorioDeViagem procurarViagem(Viagem viagem)
        throws ObjetoJaCadastradoException, ObjetoVazioException {

        RepositorioDeViagem resposta = new RepositorioDeViagem();
        procurar (viagem, resposta);
        return resposta;
    }

  /**
    * Permite o acesso por outras classes aos elementos do repositrio
    *
    * @return   Array com todos os elementos do repositrio
    */
    public Viagem[] getIterador() {
        Viagem resultado[];
        RepositorioDeViagem auxiliar;
        int quantidadeDeViagem = 0;
        int indice = 0;

        auxiliar = this;
        while(auxiliar.proximo != null) {
            quantidadeDeViagem = quantidadeDeViagem + 1;
            auxiliar = auxiliar.proximo;
        }

        // +1 para marcar com null o final do array
        resultado = new Viagem[quantidadeDeViagem + 1];

        if (quantidadeDeViagem != 0) {
            auxiliar = this;
            while(auxiliar.proximo != null) {
                resultado[indice] = auxiliar.viagem;
                auxiliar = auxiliar.proximo;
                indice = indice + 1;
            }
        }

        return resultado;
    }

/*
    public static void main(String args[])
        throws ObjetoJaCadastradoException, ObjetoVazioException, ObjetoInexistenteException {

        RepositorioDeViagem r = new RepositorioDeViagem();
        RepositorioDeViagem s;
        Viagem v;
        Viagem[] viagens;
        int i;

        try {

            // CADASTRA 3 VIAGENS
            v = new Viagem();

            // CAUSAS DA EXCECAO ObjetoVazioException
            // r.removerViagem(v);
            // r.inserirViagem(v);

            v.setEmbarque("RECIFE"); v.setDesembarque("FORTALEZA");
            r.inserirViagem(v);
            v.setEmbarque("RECIFE"); v.setDesembarque("TIMBAUBA");
            r.inserirViagem(v);
            v.setEmbarque("RECIFE"); v.setDesembarque("PETROLINA");
            r.inserirViagem(v);

            // CAUSA DA EXCECAO ObjetoInexistenteException
            // v.setDesembarque("TOKIO"); r.removerViagem(v);

            // CAUSA DA EXCECAO ObjetoJaCasdastradoException
            // r.inserirViagem(v);


            // MOSTRA AS VIAGENS NAVEGANDO PELO REPOSITORIO RESULTADO DE UMA PROCURA
            v = new Viagem();
            v.setEmbarque("RECIFE");
            s = r.procurarViagem(v);
            while (s.proximo != null) {
                System.out.println(s.viagem.getEmbarque() + " -> " + s.viagem.getDesembarque());
                s = s.proximo;
            }

            // CADASTRA MAIS 2 VIAGENS
            v = new Viagem();
            v.setEmbarque("GARANHUNS"); v.setDesembarque("RECIFE");
            r.inserirViagem(v);
            v.setEmbarque("CARUARU"); v.setDesembarque("RECIFE");
            r.inserirViagem(v);

            // MOSTRA AS VIAGENS UTILIZANDO getIterador()
            System.out.println();
            viagens = r.getIterador();
            for (i=0; i<5; i++) {
                System.out.println(viagens[i].getEmbarque() + " -> " + viagens[i].getDesembarque());
            }

            // REMOVE A PRIMEIRA E A ULTIMA DA LISTA
            v = new Viagem();
            v.setEmbarque("RECIFE"); v.setDesembarque("FORTALEZA");
            r.removerViagem(v);
            v.setEmbarque("CARUARU"); v.setDesembarque("RECIFE");
            r.removerViagem(v);

            // MOSTRA AS VIAGENS UTILIZANDO getIterador()
            System.out.println();
            viagens = r.getIterador();
            for (i=0; i<3; i++) {
                System.out.println(viagens[i].getEmbarque() + " -> " + viagens[i].getDesembarque());
            }

        }

        catch (ObjetoVazioException excecao) {
            System.out.println(excecao.getMensagem());
        }

        catch (ObjetoInexistenteException excecao) {
            System.out.println(excecao.getMensagem());
        }

        catch (ObjetoJaCadastradoException excecao) {
            System.out.println(excecao.getMensagem());
        }

    }
*/

}

